;******************************************************************************
;                      S S T V A I N . A S M
;******************************************************************************
;This module checks for new A/D input data in the circular buffer "Sample_Que"
;and then processes the new data.  The analog data is processed and demodulated
;for further decoding.
;------------------------------------------------------------------------------
;                     C O N S T A N T    E Q U A T E S
;------------------------------------------------------------------------------
; NCO rate frequency increment = SampleFreq/65536 = 10893/65536 = .166213989 Hz
INC1700         .equ    10228   ;incr value for 1700 HZ =  1700/.166213989
MAX_FREQ        .equ    13837   ;incr value for 2300 HZ =  2300/.166213989
MIN_FREQ        .equ    6016    ;incr value for 1000 HZ =  1000/.166213989
;
AGC_LIMIT       .equ    360
;
PEAK_SLEW       .equ    100
PEAK_LEAK       .equ    15
PEAK_MAX_LIM    .equ    32000
PEAK_MIN_LIM    .equ    -1000
PK_PK_LIM       .equ    10924   ;pll lock min noise detect threshold
LOCK_THRESH     .equ    2731    ;pll lock min level detect threshold
;
;FREQ_CONST      .equ    6000   ;used for experimental wide filter set
;PHASE_CONST     .equ    24000

FREQ_CONST      .equ    3000    ;DPLL loop constants
PHASE_CONST     .equ    12000
;
;       COSINE offset value into SIN_TABLE
PHASE90         .equ    16128   ;(256/4-1)*256 SIN table offset for COS
;
ADLIMIT         .equ    21845   ;Receive signal peak indicator limit
                                ;2^15*2.0/3 == 2.0 volt Peaks
ONESHOT_LEN     .equ    20      ;signal peak indicator display time
                                ;20 * .005 = .1 SEC
;==============================================================================
;      ========          I n i t _ A i n ( )     ==========
;==============================================================================
Init_Ain:                       ;initialize variables and constants
        lrlk    AR6,Sample_Que  ;init circular A/D & D/A buffer pointers
        lrlk    AR5,Sample_Que
        lrlk    AR4,Sample_Que+1
;
        lalk    INC1700         ;nco center frequency
        sacl    ncofreq
;
        lalk    ADLIMIT
        sacl    AD_Limit
;
        lalk    MIN_FREQ
        sacl    MIN_freq
        lalk    MAX_FREQ
        sacl    MAX_freq
;
        lalk    AGC_LIMIT
        sacl    AGC_limit
;
        lalk    PEAK_SLEW
        sacl    Peak_slew
;
        lalk    PEAK_LEAK
        sacl    Peak_leak
;
        lalk    PEAK_MAX_LIM
        sacl    Peak_max_lim
;
        lalk    PEAK_MIN_LIM
        sacl    Peak_min_lim
;
        lalk    PK_PK_LIM
        sacl    Pk_Pk_lim
;
        lalk    LOCK_THRESH
        sacl    Lock_thresh
;
        lalk    FREQ_CONST
        sacl    Freq_const
        lalk    PHASE_CONST
        sacl    Phase_const
;
        zac
        sacl    AGC_Sum_h
        sacl    AGC_Sum_l
        sacl    AGC_Ave
        sacl    Htimer
        sacl    Hdelay
        sacl    Hlevel
;
;Load FIR Filter Coefficients into B0 Page memory
        cnfd                    ;map B0 to data memory
        larp    AR0             ;ARP = AR0 = delay line pointer
        lrlk    AR0,IBPFIRCOEF+BPFIR_LENGTH-1
        rptk    BPFIR_LENGTH-1  ;load I term BP FIR filter coefficients
        blkp    IBPFCOEF,*-     ;loads reversed order coef
        cnfp                    ;map B0 to program memory
;
        cnfd                    ;map B0 to data memory
        larp    AR0             ;ARP = AR0 = delay line pointer
        lrlk    AR0,QBPFIRCOEF+BPFIR_LENGTH-1
        rptk    BPFIR_LENGTH-1  ;load Q term BP FIR filter coefficients
        blkp    QBPFCOEF,*-     ;loads reversed order coef
        cnfp                    ;map B0 to program memory
;
        ret
;..............................................................................
;==============================================================================
;        ========     S e r v i c e _ A i n ( )     ==========
;==============================================================================
;void Service_Ain()
;{
;  if( AR5 != AR6 ){            // if receive Sample_Que is not empty
;     Temp1 = Sample_Que[ AR5++ ];
;     if( abs( Temp1 ) > AD_Limit ){  //indicate clipping
;        Flags.PEAK_DET = TRUE;
;        Timer2 = ONESHOT_LEN;
;        LED7on();
;        Temp1 = Prev_sample;   // use previous sample if over ranged
;     }else{
;        Prev_sample = Temp1;
;     }
;     if( AR5 > Sample_Que + SAMPQUESIZE-1 )
;        AR5 = Sample_Que;
;     AGC_calc( Temp1 );       // perform AGC on input sample
;     ProcessInput( Temp1 );   // process new input sample
;  }
;  else{                       // here if no new sample to process
;     if( Flags.PEAK_DET ){
;        if( !Timer2 ){
;           Flags.PEAK_DET = FALSE;
;           LED7off();
;        }
;     }
;  }
;}
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Service_Ain:                    ;service input samples from A/D
        larp    AR5             ;ARP = Sample_Que tail pointer
        sar     AR6,Temp0       
        lar     AR0,Temp0       ;Temp0 = head
        cmpr    CMEQ            ;see if head == tail   TC = (AR6==AR5)
        bbnz    SA_Emty         ;br if Sample_Que is empty
        larp    AR5             ;get sample
;
        lac     *+              ;T = 16 bit A/D input value(+/- 32767)
        sacl    Temp1
;
        abs
        sub     AD_Limit
        blz     SA_1            ;br if |signal| less than the threshold limit
        lac     Flags
        ork     PEAK_DET        ;latch peak event
        sacl    Flags
        lack    ONESHOT_LEN     ;start LED7 pulse stretcher
        sacl    Timer2
        call    LED7on
        lac     Prev_sample
        sacl    Temp1           ;use previous sample if new one is overanged
        b       SA_2
;
SA_1:   lac     Temp1
        sacl    Prev_sample
SA_2:   lrlk    AR0,Sample_Que+SAMPQUESIZE-1    ;check for wrap around
        cmpr    CMGT            ;TC = (AR5>AR0)
        bbz     SA_3            ;br if no wrap
        lrlk    AR5,Sample_Que  ;else wrap it around
SA_3:
        call    AGC_calc        ;Do the AGC on Temp1 sample
        call    ProcessInput    ;go process next A/D input sample in Temp1
        ret
SA_Emty:
        bit     Flags,T_PEAK_DET
        bbz     SA_exit         ;if peak det LED is on then
        lac     Timer2          ;check timeout
        bnz     SA_exit
        lac     Flags
        andk    ~PEAK_DET
        sacl    Flags
        call    LED7off
SA_exit:
        ret
;
;==============================================================================
;             =======   A G C _ c a l c ( Temp1 )     =======
;==============================================================================
;This routine is called every A/D sample and calculates a new AGC gain value.
;
;void AGC_calc(Temp1)
;   DeltaNew = OldInput - NewInput;
;   if( ( DeltaNew ^ DeltaOld ) & BIT15 ){      // if sign changed
;      OldInput = abs( OldInput );      // OldInput is peak signal
;      AGC_Sum = AGC_Sum + OldInput - AGC_Ave;
;      AGC_Ave = AGC_Ave/8;
;   }
;   DeltaOld = DeltaNew;
;   OldInput = Temp1;
;
;   AGCdelay[ 0 ] = Temp1;              // put latest sample in delay line
;   Temp1 = AGCdelay[ AGC_LENGTH-1 ];   // get oldest sample out for processing
;   for( i=AGC_LENGTH-2; i >= 0; i-- ){ // shift everybody down
;      AGCdelay[ i+1 ] = AGCdelay[ i ];
;   }
;   Temp1 = Temp1/AGC_Ave;              // scale sample by AGC gain
;}
AGC_calc:
        lac     OldInput
        sub     Temp1
        sacl    DeltaNew        ;DeltaNew = OldInput - NewInput
        xor     DeltaOld        ;see if derivative sign changed
        sacl    Temp0
        bit     Temp0,15        ;see if sign bit has changed
        bbz     Ac_1            ;br if not at signal peak
        lac     OldInput
        abs
        sacl    OldInput
;
        zals    AGC_Sum_l
        addh    AGC_Sum_h
        sub     AGC_Ave         ;sub old average
        add     OldInput        ;add another sample
        sach    AGC_Sum_h
        sacl    AGC_Sum_l
        sfr                     ;running average over 8 samples
        sfr
        sfr
        sacl    AGC_Ave
Ac_1:
        lac     DeltaNew        ;update AGC delay lines
        sacl    DeltaOld
        lac     Temp1
        sacl    OldInput
;
        larp    AR0
        lrlk    AR0,AGCdelay
        sacl    *
        adrk    AGC_LENGTH-1
        lac     *-
        sacl    Temp1
        rptk    AGC_LENGTH-2
        dmov    *-
;
        lac     AGC_Ave         ; see if enough signal to AGC
        sub     AGC_limit
        bgez    Ac_2
        call    LED2on
        lac     Flags
        andk    ~AGC_DET
        sacl    Flags           ;Signal too weak to AGC
        b       Ac_3
Ac_2:   call    LED2off
        lac     Flags
        ork     AGC_DET
        sacl    Flags           ;Signal enough to AGC
;
Ac_3:   lac     AGC_Ave
        adlk    0200h
        sacl    Temp0           ;Temp0 = AGC_Ave + (1/128)
        lalk    0400h,15        
        rptk    14
        subc    Temp0
        sacl    Temp0           ;temp0 = (1/256)/Temp0;
;
        lt      Temp1           ;T = 16 bit input value(+/- 32767)
        mpy     Temp0
        pac
        sach    Temp1,4         ;multiply input by AGCgain value
        ret
;
;==============================================================================
;      =======   P r o c e s s I n p u t ( input )     =======
;==============================================================================
;This routine is called every A/D sample and processes the AGC'ed input
;sample.  The input is bandpass filtered into two channels(I andQ) using,
;a normal bandpass filter for the Iterm and an identical bandpass filter with
;a hilbert transform for the Qterm.  A complex nco is implemented with a
;phase accumulator(ncophzacc) that is added to by a phase error(Phzerr) term
;and a frequency term (ncofreq). 
;
;void ProcessInput( input )
;{
;   BPFdelay[ 0 ] = input;      // do FIR calculations to get I and Q
;   acc = 0;
;   for( i=BPFIR_LENGTH-1; i >= 0; i-- ){
;      acc += BPFdelay[ i ] * IBPFIRCOEF[ i ];
;   }
;   Iterm = acc;                // Iterm =  I bandpass output data 
;   for( i=BPFIR_LENGTH-1; i >= 0; i-- ){
;      acc += BPFdelay[ i ] * QBPFIRCOEF[ i ];
;      BPFdelay[ i+1 ] = BPFdelay[ i ];         // do the shuffle
;   }
;   Qterm = acc;                // Qterm =  Q hilbert bandpass output data 
;
;   ncophzacc = ncophzacc + ncofreq + Phzerr;   // update new phase
;   nco_sin = SIN_TABLE[ ncophzacc>>8 ];        // create nco sin and cos
;   nco_cos = SIN_TABLE[ ncophzacc>>8 + PHASE90 ];
;
;   Lock = Iterm*nco_cos + Qterm*nco_sin;       // calc lock signal
;   Check_lock();                               // see if phaselocked
;
;   Phzerr = Qterm*nco_cos - Iterm*nco_sin;     // calc phase error
;   Phzerr = Phzerr * Phase_const;              // scale phase error
;
;   ncofreq =  ncofreq + (Phzerr * Freq_const); // calc new frequency
;                                               // for phase lock
;   if( ncofreq < MIN_freq )          // clamp new frequency within bounds
;      ncofreq =  MIN_freq;
;   else
;      if( ncofreq > MAX_freq )
;         ncofreq =  MAX_freq;
;}
;..........................................................................
ProcessInput:
;$$$$$$$$$$$$$$$$
;        lac     Temp1
;        andk    0FFFCh
;        sacl    TestOut         ;TEST OUT
;$$$$$$$$$$$$$$$$
        larp    AR0             ;ARP = AR0 = delay line pointer
        lrlk    AR0,BPFdelay    ;AR0 points to start of delay line
        lac     Temp1
        sacl    *               ;store new sample at beginning of buf
;
; . . . . . .   Calculate value for I  ( normal bandpass filter )
        adrk    BPFIR_LENGTH-1  ;point to end of delay line
        mpyk    0		;initialize sum and product registers
        zac
        rptk    BPFIR_LENGTH-1  ;multiply and accumulate
        mac     (IBPFIRCOEF+B0PADDR-B0DADDR),*-
        apac                    ; include last product
        sach    Iterm           ;filter gain = 1
;
; . . . . . .   Calculate value for Q  ( Hilbert bandpass filter )
        lrlk    AR0,BPFdelay    ;AR0 points to start of delay line
        adrk    BPFIR_LENGTH-1  ;point to end of delay line
        mpyk    0		;initialize sum and product registers
        zac
        rptk    BPFIR_LENGTH-1  ;multiply and accumulate
        macd    (QBPFIRCOEF+B0PADDR-B0DADDR),*-
        apac                    ; include last product
        sach    Qterm           ;filter gain = 1
;
;$$$$$$$$$$$$$$$$
;        lac     Iterm
;        andk    0FFFCh
;        sacl    TestOut         ;TEST OUT
;$$$$$$$$$$$$$$$$
;. . . . . . . . . .  Update nco  . . . . . . . . . . . .
        lac     ncophzacc
        add     ncofreq       ;add phase
        add     Phzerr
        sacl    ncophzacc
        adlk    PHASE90
        sacl    nco_cos         ;temporary save of Cos pointer
;
        rsxm
        lac     ncophzacc,8     ;get current sin phase accumulator value
        sach    nco_sin         ;SIN table_ptr=top 8 msbits
        lac     nco_sin
        adlk    SIN_TABLE       ;add phase to SIN table start address
        tblr    nco_sin         ;get next SIN value from table
;
        lac     nco_cos,8       ;get current cos phase accumulator value
        sach    nco_cos         ;COS table_ptr=top 8 msbits
        lac     nco_cos
        adlk    SIN_TABLE       ;add phase to SIN table start address
        tblr    nco_cos         ;get next COS value from table
        ssxm
; create lock signal  Lock = Iterm*cos(w) + Qterm*sin(w)
;
        zac
        lt      Iterm           ;T reg = Iterm
        mpy     nco_cos         ;P = Iterm*nco_cos
        lt      Qterm           ;T reg = Qterm
        mpya    nco_sin         ; acc = Iterm*nco_cos, P = Qterm*nco_sin
        apac
        sach    Temp0
;$$$$$$$$$$$$$$$$
;        lac     Temp0
;        andk    0FFFCh
;        sacl    TestOut         ;TEST OUT
;$$$$$$$$$$$$$$$$
        call    Check_lock
;
; create phase error signal  Phzerr = -Iterm*sin(w) + Qterm*cos(w)
;
        lac     Iterm
        neg
        sacl    Temp0
        zac
        lt      Temp0           ;T reg = -Iterm
        mpy     nco_sin         ;P = -Iterm*nco_sin
        lt      Qterm           ;T reg = Qterm
        mpya    nco_cos         ; acc = -Iterm*nco_sin, P = Qterm*nco_cos
        apac
        sach    Phzerr
;
;$$$$$$$$$$$$$$$$
;        lac     Phzerr
;        andk    0FFFCh
;        sacl    TestOut         ;TEST OUT
;$$$$$$$$$$$$$$$$
        lt      Phzerr
        mpy     Phase_const
        pac
        sach    Phzerr          ;scale phase error
        mpy     Freq_const
        pac
        sach    Temp0           ;scale frequency error
;
        lac     ncofreq       ;update ncofreq( frequency )
        add     Temp0
        sacl    ncofreq
        sub     MIN_freq
        bgz     PI_1            ;br if greater than MIN freq limit
        lac     MIN_freq        ;clamp to max freq
        b       PI_3
PI_1:   add     MIN_freq
        sub     MAX_freq        ;br if less than MAX freq limit
        blz     PI_2    
        lac     MAX_freq        ;clamp to min freq
        b       PI_3
PI_2:   add     MAX_freq
PI_3:   sacl    ncofreq
;. . . . . . . . . . . . . . . . . . . . . . . .
;$$$$$$$$$$$$$$$$
;        lac     ncofreq
;        lac     nco_sin
;        andk    0FFFCh
;        sacl    TestOut         ;TEST OUT
;$$$$$$$$$$$$$$$$
;
        ret
;==============================================================================
;             =======   C h e c k _ l o c k  ( Temp1 )     =======
;==============================================================================
;This routine is called to determine if the PLL is locked. The peak to peak
;value of a signal is found and if the pk-pk value is less than a specified
;limit and the overall value is above a specified limit then the PLL is
;locked. The input sample is in "Temp0"
;
;void Check_lock( Temp0 )
;   if( Temp0 > Peak_max )      // if greater than current max level
;      Peak_max += Peak_slew;
;   else
;      Peak_max -= Peak_leak;
;   if( Temp0 < Peak_min )      // if less than current min level
;      Peak_min -= Peak_slew;
;   else
;      Peak_min += Peak_leak;
;   if( Peak_max > Peak_max_lim )      // clamp to peak limits
;      Peak_max = Peak_max_lim;
;   if( Peak_min < Peak_min_lim )      // clamp to peak limits
;      Peak_min = Peak_min_lim;
;
;   Temp0 = Peak_max - Peak_min;
;   Temp1 = (Peak_max + Peak_min) / 2;
;   if( (Temp0 <= Pk_Pk_lim) && (Temp1 >= Lock_thresh) ){
;      Flags.LOCK_DET = TRUE;
;   }
;   else{
;      Flags.LOCK_DET = FALSE;
;   }
;
;}
Check_lock:
        bit     Flags,T_AGC_DET
        bbz     Cl_7            ;if signal too low for AGC then exit
        lac     Temp0
        sub     Peak_max
        blz     Cl_1
        lac     Peak_max
        add     Peak_slew
        b       Cl_2
Cl_1:   lac     Peak_max
        sub     Peak_leak
Cl_2:   sacl    Peak_max
;
        lac     Temp0
        sub     Peak_min
        bgz     Cl_3
        lac     Peak_min
        sub     Peak_slew
        b       Cl_4
Cl_3:   lac     Peak_min
        add     Peak_leak
Cl_4:   sacl    Peak_min
;
        lac     Peak_max
        sub     Peak_max_lim
        blez    Cl_5
        lac     Peak_max_lim
        sacl    Peak_max
Cl_5:   lac     Peak_min
        sub     Peak_min_lim
        bgez    Cl_6
        lac     Peak_min_lim
        sacl    Peak_min
;
Cl_6:   lac     Peak_max
        sub     Peak_min
        sacl    Temp0           ;Temp0 = Peak_max - Peak_min
        lac     Peak_max
        add     Peak_min
        sfr
        sacl    Temp1           ;Temp1 = (Peak_max + Peak_min)/2
;$$$$$$$$$$$$$$$$
;        lac     Temp0
;        lac     Temp1
;        andk    0FFFCh
;        sacl    TestOut         ;TEST OUT
;$$$$$$$$$$$$$$$$
;
        lac     Temp0
        sub     Pk_Pk_lim
        bgz     Cl_7
        lac     Temp1
        sub     Lock_thresh
        blz     Cl_7
        lac     Flags
        ork     LOCK_DET        ;Signal PLL is locked
        sacl    Flags
        call    LED1on
        ret
Cl_7:   lac     Flags
        andk    ~LOCK_DET
        sacl    Flags           ;Signal PLL is un-locked
        call    LED1off
        ret
;==============================================================================
;                 =======   i n v _ x ( Temp0 )     =======
;==============================================================================
;   This routine finds the inverse (1/Temp0) in Q15 format.
; Temp0 = 1/Temp0;
;------------------------------------------------------------------------------
inv_x:
    lac     Temp0
    bnz     inv_x1
    lalk    07FFFh      ;check for divide by zero case
    b       inv_x2
inv_x1:
    lalk    07FFFh
    rptk    14
    subc    Temp0
inv_x2:
    sacl    Temp0
    ret
;==============================================================================
;.................E N D   o f   S S T V A I N . A S M ......................
;==============================================================================
