;******************************************************************************
;                      S S T V A O U T . A S M
;******************************************************************************
;This module takes the value in TXfreq and converts it into the proper tones
;for the CW and SSTV signal.  A new D/A output sample is calculated and placed
;in the circular buffer "Sample_Que".
;==============================================================================
;      ========          I n i t _ A o u t ( )     ==========
;==============================================================================
Init_Aout:                      ;initialize variables
        cnfd                    ;map B0 to data memory
        larp    AR0             ;ARP = AR0 = delay line pointer
        lrlk    AR0,LPFIRCOEF+LPFIR_LENGTH-1
        rptk    LPFIR_LENGTH-1  ;load LP FIR filter coefficients
        blkp    LPFCOEF,*-      ;loads reversed order coef
        cnfp                    ;map B0 to program memory
;
        zac
        sacl    TXfreq
        sacl    CW_Sum_h
        sacl    CW_Sum_l
        sacl    CW_Ave
        ret
;..............................................................................
;==============================================================================
;       =========      S e r v i c e _ A o u t ( )    =========
;==============================================================================
;Put samples into the D/A converter circular buffer for transmitting.
;void Service_Aout()
;{
;  if( AR4 != AR5 ){            // if transmit Sample_Que is not full
;     Sample_Que[ AR4++ ] = OutBuf;
;     if( AR4 > Sample_Que + SAMPQUESIZE-1 )
;        AR4 = Sample_Que;
;     if( Flags.TRANSMITON ){           // if transmit mode is active
;        if( Flags.CWID_ACTIVE ){       
;           CW_Sum = CW_Sum + TXfreq - CW_Ave;  //TXfreq is actually
;           CW_Ave = CW_Ave/64;                 //the amplitude modulation
;           TXphzacc = TXphzacc + CWFREQ;       // update new phase
;           Temp1 = SIN_TABLE[ TXphzacc>>8 ];   // create TX sin values
;           OutBuf = CW_Ave*Temp1;
;        }
;        }else{
;           LPFdelay[ 0 ] = TXfreq;     // do Low pass FIR calculation
;           acc = 0;
;           for( i=LPFIR_LENGTH-1; i >= 0; i-- ){
;              acc += LPFdelay[ i ] * LPFIRCOEF[ i ];
;              LPFdelay[ i+1 ] = LPFdelay[ i ];  // do the shuffle
;           }
;           Temp0 = acc;
;           TXphzacc = TXphzacc + Temp0;        // update new phase
;           OutBuf = SIN_TABLE[ TXphzacc>>8 ];  // create TX sin values
;        }
;     }else{
;        OutBuf = nco_sin;                      //if not xmitting
;        TXphzacc = 0;                          // send receiver nco
;     }
;  }
;}
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
Service_Aout:                   ;service sending of output samples to D/A
        larp    AR4             ;ARP = Sample_Que xmit head pointer
        sar     AR5,Temp0       ;
        lar     AR0,Temp0       ;Temp0 = receive tail
        cmpr                    ;see if head == tail
        bbnz    SAo_6           ;br to exit if Sample_Que is full
        lac     OutBuf
        andk    0FFFCh
        sacl    *+              ;put next output data in Sample_Que and
                                ;inc head pointer
        lrlk    AR0,Sample_Que+SAMPQUESIZE-1    ;check for wrap around
        cmpr    CMGT            ;TC= (AR4>AR0)
        bbz     SAo_1           ;br if no wrap
        lrlk    AR4,Sample_Que  ;else wrap it around
SAo_1:
        bit     Flags,T_TRANSMITON
        bbz     SAo_5           ;if transmiter is not on
        larp    AR0             ;ARP = AR0 = delay line pointer
        lrlk    AR0,LPFdelay    ;AR0 points to start of delay line
        lac     TXfreq
        sacl    *               ;store new sample at beginning of buf
;
        bit     Flags,T_CW_ACTIVE
        bbnz    SAo_4          ;br if cw sending
; . . . . . .   Calculate  ( SSTV data lowpass filter )
        adrk    LPFIR_LENGTH-1  ;point to end of delay line
        mpyk    0		;initialize sum and product registers
        zac
        rptk    LPFIR_LENGTH-1  ;multiply and accumulate
        macd    (LPFIRCOEF+B0PADDR-B0DADDR),*-
        apac                    ; include last product
        sach    Temp0,1         ;filter gain = 2*.5
;
        lac     TXphzacc
        add     Temp0           ;add phase increment (frequency)
        sacl    TXphzacc
        rsxm
        lac     TXphzacc,8      ;get current sin phase accumulator value
        sach    Temp0           ;SIN table_ptr=top 8 msbits
        lac     Temp0
        adlk    SIN_TABLE       ;add phase to SIN table start address
        tblr    OutBuf          ;get next SIN value from table
        ssxm
        b       SAo_6
SAo_4:
; . . . . . .   Calculate  ( CW data shaping lowpass filter )
        zals    CW_Sum_l
        addh    CW_Sum_h
        sub     CW_Ave          ;sub old average
        add     TXfreq          ;add another sample
        sach    CW_Sum_h
        sacl    CW_Sum_l
        sfl
        sfl
        sfl
        sach    CW_Ave,7        ;running average over 64 samples
;
        lac     TXphzacc
        adlk    CWFREQ          ;add phase increment for CW frequency
        sacl    TXphzacc
        rsxm
        lac     TXphzacc,8      ;get current sin phase accumulator value
        sach    Temp1           ;SIN table_ptr=top 8 msbits
        lac     Temp1
        adlk    SIN_TABLE       ;add phase to SIN table start address
        tblr    Temp1           ;get next SIN value from table
        ssxm
        lt      Temp1           ;T reg = sin amplitude
        mpy     CW_Ave          ;P = sin(CWFREQ) * filtered amplitude
        pac
        sach    OutBuf
        b       SAo_6
SAo_5:  zac
        sacl    TXphzacc
        sacl    CW_Sum_h
        sacl    CW_Sum_l
        sacl    CW_Ave
        lac     nco_sin         ;echo nco frequency if not transmitting
        sacl    OutBuf
SAo_6:  ret

;...............E N D   o f   S S T V A O U T . A S M ......................
