;******************************************************************************
;                         S S T V I N . A S M
;******************************************************************************
;   This module waits for a SSTV data byte to be received by the UART
;and processes it into "TXfreq" for use by the SSTVAOUT.ASM module.
;
BYTELIMIT       .equ    25      ; number of UART bytes in BYTETIMELIMIT
                                ;needed to be valid transmit data
TX_VIS0         .equ    125
TX_HSYNC        .equ    126
TX_VIS1         .equ    127
;
BYTETIMELIMIT   .equ    4       ; .005 * 4 = .02 sec
DOT_TIME        .equ    12      ; DOT_TIME = 240/wpm
DASH_TIME       .equ    3*DOT_TIME
CWID_DELAY      .equ    50
CW_AMPLITUDE    .equ    20000

;==============================================================================
;      ========          I n i t _ S S T V i n ( )     ==========
;==============================================================================
;   if( TNC_INPUT.pin18 == +5V )
;      Flags.PTT1_LOGIC = TRUE;
;   else
;      Flags.PTT1_LOGIC = FALSE;
;   if( TNC_INPUT.pin16 == +5V )
;      Flags.PTT2_LOGIC = TRUE;
;   else
;      Flags.PTT2_LOGIC = FALSE;
;   PTT1_off();
;   PTT2_off();
;   if( R2 ){
;      Flags.RADIO_PORT = 1;
;      Sel_Input( REC_IN_2 );
;   }else{
;      Flags.RADIO_PORT = 0;
;      Sel_Input( REC_IN_1 );
;   }
;   switch( Gx )
;      case G1:
;         Sel_Gain( GAIN0 );
;         break;
;      case G1:
;         Sel_Gain( GAIN1 );
;         break;
;      case G1:
;         Sel_Gain( GAIN3 );
;         break;
;      case G1:
;         Sel_Gain( GAIN4 );
;         break;
;      case G1:
;         Sel_Gain( GAIN5 );
;         break;
;      case G1:
;         Sel_Gain( GAIN6 );
;         break;
;      default:
;         Sel_Gain( GAIN2 );
;         break;
;}
Init_SSTVin:                    ;initialize variables
        in      Temp1,TNC_INPUT ;read PTT polarity jumper value
        lac     Temp1
        andk    RDO             ;chk PTT1
        bnz     In_K_1          ;br if port1 PTT is NEG logic
        lac     Flags           ;else is POS logic
        ork     PTT1_LOGIC
        b       In_K_2
In_K_1: lac     Flags
        andk    ~PTT1_LOGIC
        b       In_K_2
In_K_2: sacl    Flags
        lac     Temp1
        andk    XCLKO           ;chk PTT2
        bnz     In_K_3          ;br if port2 PTT is NEG logic
        lac     Flags           ;else is POS logic
        ork     PTT2_LOGIC
        b       In_K_4
In_K_3: lac     Flags
        andk    ~PTT2_LOGIC
In_K_4: sacl    Flags
;
        call    PTT1_off
        call    PTT2_off
;
#ifdef  R2
        lac     Flags
        ork     RADIO_PORT
        sacl    Flags
        lack    REC_IN_2
        sacl    Temp0
        call    Sel_Input
#else                           ;default port is PORT 1
        lac     Flags
        andk    ~RADIO_PORT
        sacl    Flags
        lack    REC_IN_1
        sacl    Temp0
        call    Sel_Input
#endif
;
        lack    GAIN2           ;default gain is GAIN2 (x4)
#ifdef  G1
        lack    GAIN0
#endif
#ifdef  G2
        lack    GAIN1
#endif
#ifdef  G8
        lack    GAIN3
#endif
#ifdef  G16
        lack    GAIN4
#endif
#ifdef  G32
        lack    GAIN5
#endif
#ifdef  G64
        lack    GAIN6
#endif
        sacl    Temp0
        call    Sel_Gain
;
        zac
        sacl    Byte_counter
        lack    BYTETIMELIMIT
        sacl    Timer2reload
        lalk    Byte_check      ;initialize state vector
        sacl    SSTVin_Vector
;
        lalk    TXFREQSCALE
        sacl    TXfreqscale
;
        ret
;.............................................................................
;========================================================================
;                 S e r v i c e _ S S T V i n ( )
;========================================================================
;void Service_SSTVin()
:{
;   if( !UART_EMPTY ){          // See if data in UART receive buffer
;      if( !UART_ERROR ){       // See if error in UART receive buffer
;         Temp0 = UART INPUT BYTE;
;         if( Flags.TRANSMITON && !Flags.CW_ACTIVE )
;            calc_tones(Temp0);
;         Byte_counter++;
:      }
;      }else{
;         Temp0 = UART INPUT BYTE;
;         Byte_counter = 0;
;      }
;   }else{
;      if( !Timer2 ){
;         Timer2 = TIMER2RELOAD;
;         SSTVin_Vector();      // call state routines
;      }
;   }
;}
;
Service_SSTVin:
        lalk    UART_SEL_LSR
        sacl    Temp0
        out     Temp0,UART_CTRL
        in      Temp0,UART_READ
        lack    UART_LSR_RBF
        and     Temp0
        bz      SVi_1            ;br if UART receive buffer empty
        lack    (UART_LSR_FE|UART_LSR_PE|UART_LSR_OE|UART_LSR_BRK|UART_LSR_FERR)
        and     Temp0
        bnz      SVi_3           ;br if UART receive error
;
        lalk    UART_SEL_RX
        sacl    Temp0
        out     Temp0,UART_CTRL
        in      Temp0,UART_READ  ;Temp0 = latest byte from UART
        lac     Temp0            ;just get lower byte
        andk    0FFh
        sacl    Temp0
        bit     Flags,T_TRANSMITON
        bbz     SVi_4            ;if transmiter is not on
        bit     Flags,T_CW_ACTIVE
        bbnz    SVi_4            ;if CW ID in progress
        call    Calc_tones
SVi_4:  INC(Byte_counter)
        b       SVi_2           ;exit
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
SVi_3:                          ;Here if UART receiver error
        lalk    UART_SEL_RX     ;clear out bad byte
        sacl    Temp0
        out     Temp0,UART_CTRL
        in      Temp0,UART_READ 
        zac
        sacl    Byte_counter
SVi_1:                          ;Here if UART receiver is empty or error
        lac     Timer2
        bnz     SVi_2           ;exit if Timer2 hasn't expired
        zals    Timer2reload
        sacl    Timer2
        lac     SSTVin_Vector
        cala                    ;call state vector    
SVi_2:  ret
;========================================================================
;                 B y t e _ c h e c k ( )
;========================================================================
; This state routine checks to see if a minimum number of bytes are being
; received by the UART for transmission.  If so then the transmit mode is
; entered and the transmision of the data is allowed to occur.  If the
; transmit mode was active and no more bytes are received from the UART
; then it is assumed the transmision is over and the Morse ID state is
; begun.
;
;void Byte_check( void )
;{
;   if( Byte_counter > BYTELIMIT ){
;      Transmit_On();
;   }else{
;      if( Flags.TRANSMITON ){
;         Timer2reload = 0;
;         Timer2 = CWID_DELAY;
;         cw_idptr = 0;
;         Flags.CW_ACTIVE = TRUE;
;         SSTVin_Vector = send_msb;
;      }
;   }
;   Byte_counter = 0;
;}
Byte_check:
        lac     Byte_counter
        subk    BYTELIMIT
        blz     Bc_1
        call    Transmit_On
        b       Bc_2
Bc_1:
        bit     Flags,T_TRANSMITON
        bbz     Bc_2            ;jmp if transmiter is not on
        zac                     ;Send CW ID string and then turn off xmitter
        sacl    cw_idptr
        sacl    TXfreq
        sacl    Timer2reload
        lack    CWID_DELAY
        sacl    Timer2
        lac     Flags
        ork     CW_ACTIVE
        sacl    Flags
        lalk    send_msb        ;set next state
        sacl    SSTVin_Vector
Bc_2:
        zac
        sacl    Byte_counter
        ret
;========================================================================
;                 C a l c _ t o n e s ( Temp0 )
;========================================================================
; This routine calculates the new output frequency depending on the
;   value of Temp0.
;void Calc_tones( Temp0 )
;{
;   switch( Temp0 ){
;      case TX_VIS0:            //125
;         TXfreq = VIS0FREQ;    // 1100 Hz
;         break;
;      case TX_HSYNC:           //126
;         TXfreq = HSYNCFREQ;   // 1200 Hz
;         break;
;      case TX_VIS1:            //127
;         TXfreq = VIS1FREQ;    // 1300 Hz
;         break;
;      default:                 // 63 = 2300Hz ,  0 = 1500Hz
;         TXfreq = (TXFREQSCALE*Temp0)/8 + BLK_FREQ;
;         break;
;   }
;}
Calc_tones:
        lac     Temp0
        xork    TX_HSYNC
        bnz     Ct_2            ;case HSYNC:
        lalk    HSYNCFREQ
        sacl    TXfreq
        b       Ct_exit
Ct_2:
        lac     Temp0
        xork    TX_VIS1
        bnz     Ct_3            ;case VIS1:
        lalk    VIS1FREQ
        sacl    TXfreq
        b       Ct_exit
Ct_3:
        lac     Temp0
        xork    TX_VIS0
        bnz     Ct_4            ;case VIS0:
        lalk    VIS0FREQ
        sacl    TXfreq
        b       Ct_exit
Ct_4:   
;         TXfreq = (TXFREQSCALE*Temp0)/8 + BLK_FREQ;
        lalk    BLK_FREQ
        sacl    Temp1
        lac     Temp0
        andk    03Fh
        sacl    Temp0
        lt      Temp0
        mpy     TXfreqscale
        pac
        sfr                     ;divide by 2 because of mpy auto shift
        sfr                     ;then divide by 8
        sfr
        sfr
        add     Temp1
        sacl    TXfreq
Ct_exit:
        ret
;========================================================================
;                     s e n d _ m s b ( )
;========================================================================
;void send_msb(void)
;{
;   Temp0 = CW_ID_STRING[ cw_idptr ];
;   Temp0 = Temp0>>8;           //get msb
;   if( Temp0 ){
;      cw_word = ASCII_CW_TBL[ Temp0 ];
;      cw_return_state = send_lsb;
;      SSTVin_Vector = Send_char;
;      cw_idptr++;
;   }
;   else{
;      Timer2reload = BYTETIMELIMIT;
;      SSTVin_Vector = Byte_Check;
;      Transmit_Off();
;      Flags.CW_ACTIVE = FALSE;
;   }
;}
send_msb:
        zals    cw_idptr
        adlk    CW_ID_STRING
        tblr    Temp0           ;Temp0 = CW_ID_STRING[ cw_idptr ];
        lac     Temp0,8         ;Temp0 = Temp0>>8;
        sach    Temp0
        zals    Temp0
        bz      sm_1
        adlk    ASCII_CW_TBL
        tblr    cw_word         ;cw_word = ASCII_CW_TBL[ Temp0 ];
        lalk    send_lsb
        sacl    cw_return_state ;cw_return_state = send_lsb;
        lalk    send_char
        sacl    SSTVin_Vector   ;SSTVin_Vector = Send_char;
        b       sm_2    
sm_1:
        lac     Flags
        andk    ~CW_ACTIVE
        sacl    Flags
        call    Transmit_Off
        lack    BYTETIMELIMIT
        sacl    Timer2reload
        lalk    Byte_check
        sacl    SSTVin_Vector   ;SSTVin_Vector = Byte_check;
sm_2:
        ret
;========================================================================
;                     s e n d _ l s b ( )
;========================================================================
;void send_lsb(void)
;{
;   Temp0 = CW_ID_STRING[ cw_idptr ];
;   Temp0 = Temp0 & 0x0F;       //get lsb
;   if( Temp0 ){
;      cw_word = ASCII_CW_TBL[ Temp0 ];
;      cw_return_state = send_msb;
;      SSTVin_Vector = Send_char;
;   }
;   else{
;      Timer2reload = BYTETIMELIMIT;
;      SSTVin_Vector = Byte_Check;
;      Transmit_Off();
;      Flags.CW_ACTIVE = FALSE;
;   }
;}
send_lsb:
        zals    cw_idptr
        adlk    CW_ID_STRING
        tblr    Temp0           ;Temp0 = CW_ID_STRING[ cw_idptr ];
        zals    Temp0
        andk    0FFh             ;Temp0 = Temp0 & 0x0FF
        sacl    Temp0
        bz      sl_1
        adlk    ASCII_CW_TBL
        tblr    cw_word         ;cw_word = ASCII_CW_TBL[ Temp0 ];
        lalk    send_msb
        sacl    cw_return_state ;cw_return_state = send_lsb;
        lalk    send_char
        sacl    SSTVin_Vector   ;SSTVin_Vector = Send_char;
        INC( cw_idptr )
        b       sl_2
sl_1:
        lac     Flags
        andk    ~CW_ACTIVE
        sacl    Flags
        call    Transmit_Off
        lack    BYTETIMELIMIT
        sacl    Timer2reload
        lalk    Byte_check
        sacl    SSTVin_Vector   ;SSTVin_Vector = Byte_check;
sl_2:
        ret
;========================================================================
;                     s e n d _ c h a r  ( )
;========================================================================
;void send_char(void)
;{
;   switch( cw_word & 0xC000 ){         // get two msbit's
;      case 0x0000:
;         SSTVin_Vector = cw_return_state;
;         break;
;      case 0x4000:                     // need to send a dot
;         Timer2reload = DOT_TIME;
;         TXfreq = CW_AMPLITUDE;
;         SSTVin_Vector = send_tone;
;         break;
;      case 0x8000:                     // need to send a dash
;         Timer2reload = DASH_TIME;
;         TXfreq = CW_AMPLITUDE;
;         SSTVin_Vector = send_tone;
;         break;
;      case 0xC000:                     // need to send a space
;         Timer2reload = 2*DOT_TIME;
;         TXfreq = 0;
;         SSTVin_Vector = send_notone;
;         break;
;   }
;   cw_word << 2;
;}
send_char:
        zals    cw_word
        andk    0C000h
        bz      sc_3
        sacl    Temp1
        xork    04000h
        bnz     sc_1
        lack    DOT_TIME        ;Timer2 = DOT_TIME;
        sacl    Timer2
        lalk    CW_AMPLITUDE    ;TXfreq = CW_AMPLITUDE;
        sacl    TXfreq
        lalk    send_tone
        sacl    SSTVin_Vector
        b       sc_4
sc_1:   zals    Temp1
        xork    08000h
        bnz     sc_2
        lack    DASH_TIME       ;Timer2 = DASH_TIME;
        sacl    Timer2
        lalk    CW_AMPLITUDE    ;TXfreq = CW_AMPLITUDE;
        sacl    TXfreq
        lalk    send_tone
        sacl    SSTVin_Vector
        b       sc_4
sc_2:                          ;==C 
        lack    DOT_TIME*2     ;Timer2 = 2*DOT_TIME;
        sacl    Timer2
        zac
        sacl    TXfreq
        lalk    send_notone
        sacl    SSTVin_Vector
        b       sc_4
sc_3:
        lac     cw_return_state
        sacl    SSTVin_Vector   ; SSTVin_Vector = cw_return_state;
sc_4:
        lac     cw_word,2
        sacl    cw_word         ; cw_word << 2;
        ret
;========================================================================
;                     s e n d _ t o n e ( )
;========================================================================
;void send_tone(void)
;{
;   TXfreq = 0;
;   SSTVin_Vector = send_notone;
;}
send_tone:
        lack    DOT_TIME        ;Timer2 = DOT_TIME;
        sacl    Timer2
        zac
        sacl    TXfreq
        lalk    send_notone
        sacl    SSTVin_Vector
        ret
;========================================================================
;                     s e n d _ n o t o n e ( )
;========================================================================
;void send_notone(void)
;{
;   SSTVin_Vector = send_char;
;}
send_notone:
        lalk    send_char
        sacl    SSTVin_Vector
        ret
;..............................................................................
;==============================================================================
;      =========          T r a n s m i t _ O n ( )     ========
;==============================================================================
;       This routine turns on the transmitter.
;
Transmit_On:
        lac     Flags
        ork     TRANSMITON
        sacl    Flags
        call    LED6on
;
        bit     Flags,T_RADIO_PORT      ;check which port to use
        bbnz    Ton_1                   ;br if Port2
        call    PTT1_on
        ret
Ton_1:  call    PTT2_on
        ret
;..............................................................................
;==============================================================================
;      =========          T r a n s m i t _ O f f ( )     ========
;==============================================================================
;       This routine turns off the transmitter.
;
Transmit_Off:
        lac     Flags
        andk    ~TRANSMITON
        sacl    Flags
        call    LED6off
        bit     Flags,T_RADIO_PORT      ;check which port to use
        bbnz    Toff_1                  ;br if Port2
        call    PTT1_off
        ret
Toff_1: call    PTT2_off
        ret
;..............................................................................
;........................................................................
;..................E N D   o f   S S T V I N . A S M ....................
;........................................................................
