;******************************************************************************
;                      S S T V O U T . A S M
;******************************************************************************
;   This module waits for a the UART channel to empty then sends back the
; current value of nco_freq in JVFAX serial format.  LED3,4,and 5 are used to
;display sync information for tuning purposes.
;
VBLACK          .equ    3
HSYNC           .equ    080h
VIS1            .equ    1
VIS0            .equ    2
VDATA           .equ    3
;nco_freq range is 1000 to 2300 Hz or in counts( 6016 to 13837 )
;nco_freq is 6.01634 * frequency in Hz.
VIS0_THRESH     .equ    6317    ;1050 Hz
DELTA_THRESH    .equ    602     ;100 Hz
;
OUTPUT_SCALE    .equ    1736    ;255/(nco_freq - BLK_FREQ)=.052978( Q15 format)
;
LED_LIMIT       .equ    3       ; .78 mSec continuous signal display threshold
;==============================================================================
;     ========          I n i t _ K o u t ( )     ==========
;==============================================================================
Init_SSTVout:                   ;initialize variables
        lalk    OUTPUT_SCALE
        sacl    Output_scale
        lalk    VIS0_THRESH
        sacl    VIS0_thresh
        lalk    DELTA_THRESH
        sacl    Delta_thresh
        lalk    BLK_FREQ
        sacl    BLK_freq
;
        zac
        sacl    LED3_counter
        sacl    LED4_counter
        sacl    LED5_counter
        ret
;..............................................................................
;==============================================================================
;                 S e r v i c e _ S S T V o u t ( )
;void Service_SSTVout(void)
;{
;   if( UART_TX == EMPTY && !Flags.TRANSMITON )
;   {
;      if( nco_freq >= BLK_freq ){      // if data is in video range
;         Temp1 = (ncofreq - BLK_freq) * Output_scale;
;         Temp1 |= VDATA;
;         LED3_counter--;
;         LED4_counter--;
;         LED5_counter--;
;      }else{                           // else data is in sync range
;         if( Flags.LOCK_DET ){         // if input is Phase locked
;            acc = nco_freq - VIS0_thresh;
;            if(  acc < 0 ){            // freq < 1050Hz
;               Temp1 = VBLACK;         // clamp to black value
;            }else{
;               acc = nco_freq - Delta_thresh;
;               if( acc < 0 ){          //  1050 >= f > 1150
;                  Temp1 = VIS0;
;                  LED4_counter--;
;                  LED5_counter--;
;                  if( LED3_counter > LED_LIMIT )
;                     LED3on();
;                  else
;                     LED3_counter++;
;               }else{
;                  acc = nco_freq - Delta_thresh;
;                  if( acc < 0 ){       //  1150 >= f > 1250
;                     Temp1 = HSYNC;
;                     LED3_counter--;
;                     LED5_counter--;
;                     if( LED4_counter > LED_LIMIT )
;                        LED4on();
;                     else
;                        LED4_counter++;
;                  }else
;                     acc = nco_freq - Delta_thresh;
;                     if( acc < 0 ){               //  1250 >= f > 1350
;                        Temp1 = VIS1;
;                        LED3_counter--;
;                        LED4_counter--;
;                        if( LED5_counter > LED_LIMIT )
;                           LED5on();
;                        else
;                           LED5_counter++;
;                     }
;                  }
;               }
;            }
;         }else{                // input in sync range but not Phase locked
;            Temp1 = VBLACK;    // so just send black value
;            LED3_counter--;
;            LED4_counter--;
;            LED5_counter--;
;         }
;      }
;      Send_To_UART( Temp1 );
;   }else{                   // UART not empty so service LED timers
;      if( LED3_counter < 0 ){
;         LED3_counter = 0;
;         LED3off();
;      }
;      if( LED4_counter < 0 ){
;         LED4_counter = 0;
;         LED4off();
;      }
;      if( LED5_counter < 0 ){
;         LED5_counter = 0;
;         LED5off();
;      }
;   }
;}
;==============================================================================
Service_SSTVout:
        lalk    UART_SEL_LSR    ;select line status register
        sacl    Temp0
        out     Temp0,UART_CTRL
        in      Temp0,UART_READ ;read line status register
        lack    UART_LSR_THRE
        and     Temp0           ;check  xmit holding register full bit
        bz      SVo_10          ;br if UART Transmit buffer not EMPTY
;
        bit     Flags,T_TRANSMITON
        bbnz    SVo_10          ;skip if transmiter is on
;
        lac     ncofreq         ;here if ready to send another UART byte
        sub     BLK_freq
        blz     SVo_1           ;jmp if not in video range
        sacl    Temp1           ;make range 0 to 255 for 1500 to 2300 Hz.
        lt      Temp1           ;send scaled data
        mpy     Output_scale
        pac
        sach    Temp1           ;Temp range is 0 to 255 for 1500 to 2300
        lac     Temp1
        ork     VDATA           ;set bit 0 and bit 1 for data
        sacl    Temp1
        b       SVo_8           ;go send the video data
SVo_1:
        bit     Flags,T_LOCK_DET
        bbz     SVo_7           ;if signal not OK then send BLACK
;
        lac     ncofreq         ; here if signal OK and not video data
        sub     VIS0_thresh
        blez    SVo_7           ;clamp to black if less than VIS0
;
        sub     Delta_thresh
        bgz     SVo_2
        lack    VIS0            ;send VIS0 bit
        sacl    Temp1
        call    Send_To_UART
        b       SVo_4           ;LED3 if in between 1050 and 1150
;
SVo_2:  sub     Delta_thresh
        bgz     SVo_3
        lack    HSYNC           ; send sync code
        sacl    Temp1
        call    Send_To_UART
        b       SVo_5           ;LED4 if in between 1150 and 1250
;
SVo_3:  sub     Delta_thresh
        bgz     SVo_7           ;send Black if above VIS1
        lack    VIS1            ;send VIS1 bit
        sacl    Temp1
        call    Send_To_UART
        b       SVo_6           ;LED5 if in between 1250 and 1350
;Frequency indicator LED services
SVo_4:
        DEC(LED4_counter)
        DEC(LED5_counter)
        lac     LED3_counter
        subk    LED_LIMIT
        blz     SVo_4a
        call    LED3on
        b       SVo_9
SVo_4a: INC(LED3_counter)
        b       SVo_9
SVo_5:
        DEC(LED3_counter)
        DEC(LED5_counter)
        lac     LED4_counter
        subk    LED_LIMIT
        blz     SVo_5a
        call    LED4on
        b       SVo_9
SVo_5a: INC(LED4_counter)
        b       SVo_9
SVo_6:
        DEC(LED3_counter)
        DEC(LED4_counter)
        lac     LED5_counter
        subk    LED_LIMIT
        blz     SVo_6a
        call    LED5on
        b       SVo_9
SVo_6a: INC(LED5_counter)
        b       SVo_9
;
SVo_7:
        lack    VBLACK          ;clamp level to black
        sacl    Temp1
SVo_8:
        DEC(LED3_counter)
        DEC(LED4_counter)
        DEC(LED5_counter)
SVo_9: 
        call    Send_To_UART
        ret
;                  check LED indicator counters if UART not empty
SVo_10: 
        lac     LED3_counter
        bgez    SVo_11
        zac
        sacl    LED3_counter
        call    LED3off
SVo_11: 
        lac     LED4_counter
        bgez    SVo_12
        zac
        sacl    LED4_counter
        call    LED4off
SVo_12:
        lac     LED5_counter
        bgez    SVo_13
        zac
        sacl    LED5_counter
        call    LED5off
SVo_13:
        ret
;-----------------------------------------------------------------------------
;==============================================================================
;     ========     S e n d _ T o _ U A R T ( )       ==========
;==============================================================================
;       This routine sends a byte in Temp1 out the DSP-93 interface UART.
;
Send_To_UART:
        lalk    UART_SEL_RX     ;select transmit holding register
        sacl    Temp0
        out     Temp0,UART_CTRL
        out     Temp1,UART_WRITE
        ret
;..............................................................................
;...................E N D   o f   S S T V K O U T. A S M ......................
;..............................................................................
